/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIException.idl"
#include "domstubs.idl"

[scriptable, uuid(e06dfaea-92d5-47f7-a800-c5f5404d8771)]
interface nsIXSLTException : nsIException {
    /**
     * The node in the stylesheet that triggered the exception.
     */
    readonly attribute nsIDOMNode styleNode;

    /**
     * The context node, may be null
     */
    readonly attribute nsIDOMNode sourceNode;
};
