/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

[scriptable, uuid(f5bf5e9b-f53f-470f-b560-0d6f4c1c98ad)]
interface nsIDOMMozActivityOptions : nsISupports
{
    readonly attribute DOMString name;
    // The |data| field can be null.
    readonly attribute jsval data;
};
